/*
A web-based forum application to show off the Niggle framework.
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

The name "Jonathan Revusky" may not be used to endorse or 
promote products derived from this software without specific prior 
written permission.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.forum.data;

import com.revusky.niggle.data.*;
import com.revusky.niggle.util.*;
import java.util.*;
import java.io.IOException;

/**
 * A holder for some convenient functions for forum-related.
 */
abstract public class ForumUtil {

    static private HierarchyTracker hierarchyTracker;

    static public MutableDataSource getForumData() throws IOException { 
        DataRegistry registry = DataRegistryImpl.getDefaultRegistry();
        return (MutableDataSource) registry.getDataSource("forum_data");
    }

    static public List getForums() throws IOException {
        return getForumData().select("forum", null);
    }

    static public Forum getForum(String forumID) throws IOException {
        return (Forum) getForumData().get("forum", forumID);
    }

    static public Record getUser(String userID) throws IOException {
        return getForumData().get("forum_user", userID);
    }

    static public Article newArticle(Object forumID, Article parent) throws IOException {
        Article article = (Article) DataRegistryImpl.getDefaultRegistry().getExemplar("forum_article");
        if (parent != null) {
            forumID = parent.get("forum_id");
            article.set("parent_id", parent.getPrimaryKey());
            String title = (String) parent.get("title");
            if (!title.toLowerCase().startsWith("re:"))
                title = "Re: " + title;
            article.set("title", title);
        }
        article.set("forum_id", forumID);
        return article;
    }

    static public void storeArticle(Article article) throws IOException {
        MutableDataSource mds = getForumData();
        mds.insert(article);
    }

    static public void newForum(String shortTitle, String longTitle, String description) 
    throws IOException {
        Forum forum = (Forum) DataRegistryImpl.getDefaultRegistry().getExemplar("forum");
        forum.set("short_title", shortTitle);
        if (!Util.hasContent(longTitle)) {
            longTitle = shortTitle;
        }
        forum.set("long_title", longTitle);
        forum.set("description", description);
        getForumData().insert(forum);
    }

    static public void newUser(String loginID, String password, String email) 
    throws IOException {
        Record user = DataRegistryImpl.getDefaultRegistry().getExemplar("forum_user");
        user.set("login_id", loginID);
        user.set("password", password);
        user.set("email", email);
        getForumData().insert(user);
    }

    static public Article getArticleByID(Object id) throws IOException {
        Integer key = null;
        if (id instanceof Integer) {
            key = (Integer) id;
        }
        else try {
            key = new Integer(id.toString());
        }
        catch (Exception e) {
            return null;
        }
        return (Article) getForumData().get("forum_article", key);
    }

    static public void sortArticlesByDate(List list) {
        Collections.sort(list, dateComparator);
    }

    static public List getChildren(Article article) throws IOException {
        if (hierarchyTracker == null) {
            hierarchyTracker = new HierarchyTracker(getForumData());
        }
        return hierarchyTracker.getChildren(article);
    }

    static public int getNumSubArticles(Article article) throws IOException {
        int result = 1;
        List children = getChildren(article);
        for (Iterator it = children.iterator(); it.hasNext();) {
            Article child = (Article) it.next();
            result += getNumSubArticles(child);
        }
        return result;
    }

    static public Article getMostRecent(Article article) throws IOException {
        Article result = article;
        Date best = (Date) article.get("date");
        List children = getChildren(article);
        for (Iterator it = children.iterator(); it.hasNext();) {
            Article art = getMostRecent((Article) it.next());
            Date d = (Date) art.get("date");
            if (d.after(best)) {
                result = art;
                best = d;
            }
        }
        return result;
    }

    static final Comparator dateComparator = new Comparator() {
            public int compare(Object o1, Object o2) {
                Date d1 = (Date) ((Record) o1) .get("date");
                Date d2 = (Date) ((Record) o1) .get("date");
                if (d1 == null && d2==null)
                    return 0;
                if (d1 == null)
                    return 1;
                if (d2 == null) 
                    return -1;
                return d1.compareTo(d2);
            }
        };
}

