/*
A web-based forum application to show off the Niggle framework.
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

The name "Jonathan Revusky" may not be used to endorse or 
promote products derived from this software without specific prior 
written permission.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.forum.data;

import com.revusky.niggle.data.*;
import java.util.*;
import java.io.IOException;

class HierarchyTracker implements DataListener {

    private MutableDataSource dataSource;
    private HashMap cache = new HashMap();;

    HierarchyTracker(MutableDataSource dataSource) throws IOException {
        this.dataSource = dataSource;
        dataSource.addDataListener(this);
    }


    public void recordInserted(DataEvent event) throws IOException {
        if (event.newRecord instanceof Article) {
            Article article = (Article) event.newRecord;
            Article parent = article.getParent();
            if (parent == null) {
                return;
            }
            Object parentKey = parent.getPrimaryKey();
            int articleID = ((Integer) article.getPrimaryKey()).intValue();
            if (parent != null) {
                int[] siblings = getChildIds(parent);
                if (siblings == null || siblings.length == 0) {
                    siblings = new int[1];
                    siblings[0] = articleID;
                }
                else {
                    int[] newlist = new int[1 + siblings.length];
                    for (int i=0; i<siblings.length; i++) {
                        newlist[i] = siblings[i];
                    }
                    newlist[siblings.length] = articleID;
                    siblings = newlist;
                }
                cache.put(parentKey, siblings);
            }
        }
    }

    public void recordDeleted(DataEvent event) throws IOException {
        if (event.oldRecord instanceof Article) {
            Article article = (Article) event.oldRecord;
            Article parent = article.getParent();
            Object parentKey = parent.getPrimaryKey();
            if (parent != null) {
                int[] siblings = getChildIds(parent);
                if (siblings == null || siblings.length == 0) {
                    // This should really be a warning condition
                }
                else {
                    ArrayList al = new ArrayList(siblings.length);
                    for (int i=0; i<siblings.length; i++) {
                        al.add(new Integer(i));
                    }
                    boolean removed = al.remove(article.getPrimaryKey());
                    if (al.isEmpty()) {
                        cache.put(parentKey, null);
                    }
                    else {
                        siblings = new int[al.size()];
                        for (int i = 0; i<siblings.length; i++) {
                            siblings[i] = ((Integer) al.get(i)).intValue();
                        }
                        if (siblings.length == 0)
                            siblings = null;
                        cache.put(parentKey, siblings);
                    }
                }
            }
        }
    }

    public void recordUpdated(DataEvent event) throws IOException {
       // I guess there is no need to do anything here.
    }

    List getChildren(Article article) throws IOException {
        synchronized(dataSource) {
            int[] ids = getChildIds(article);
            ArrayList result = new ArrayList(ids.length);
            for (int i=0; i < ids.length; i++) {
                result.add(ForumUtil.getArticleByID(new Integer(ids[i])));
            }
            return result;
        }
    }

    private int[] getChildIds(Article article) throws IOException {
        Object pkey = article.getPrimaryKey();
        if (cache.containsKey(pkey)) {
            int[] result = (int[]) cache.get(pkey);
            if (result == null) 
                result = new int[0];
            return result;
        }
        else {
            List children = getChildrenFromDataSource(article);
            int[] ids = new int[children.size()];
            for (int i=0; i<ids.length; i++) {
                Article child = (Article) children.get(i);
                ids[i] = ((Integer) child.getPrimaryKey()).intValue();
            }
            if (ids.length == 0)
                cache.put(pkey, null);
            else
                cache.put(pkey, ids);
            return ids;
        }
    }

    private List getChildrenFromDataSource(Article article) throws IOException {
        RecordFilter filter = new RecordFilter.FieldConstraint("parent_id", article.getPrimaryKey());
        List result = dataSource.select("forum_article", filter);
        ForumUtil.sortArticlesByDate(result);
        Collections.reverse(result);
        return result;
    }
}
