/*
A web-based forum application to show off the Niggle framework.
Copyright (c) 2001, Jonathan Revusky
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

Redistributions of source code must retain the above copyright notice,
this list of conditions and the following disclaimer.

Redistributions in binary form must reproduce the above copyright notice,
this list of conditions and the following disclaimer in the documentation
and/or other materials provided with the distribution.

The name "Jonathan Revusky" may not be used to endorse or 
promote products derived from this software without specific prior 
written permission.

THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

package com.revusky.forum.servlet;

import com.revusky.niggle.servlet.*;
import com.revusky.niggle.data.*;
import com.revusky.niggle.util.*;
import com.revusky.forum.data.*;
import javax.servlet.*;
import javax.servlet.http.*;
import java.io.*;
import java.util.*;
import sun.net.smtp.SmtpClient;

/**
 * The class containing the entry points to the forum servlet's
 * functionality.
 */

public class ForumServletInteraction extends ServletInteraction {

    // The obligatory constructor.

    public ForumServletInteraction(HttpServletRequest request, 
                                   HttpServletResponse response,
                                   NiggleConfig config) throws IOException {
        super(request, response, config);
    }

    public void execDefault() throws IOException {
        page = getPage("topview.nhtml");
    }

    public void execDisplayForum() throws IOException {
        page = getPage("forum.nhtml");
        String forumID = getParameter("forum_id");
        page.expose("forum", ForumUtil.getForum(forumID));
    }

    public void execDisplayArticle() throws IOException {
        page = getPage("article.nhtml");
        String articleID = getParameter("article_id");
        Article article = ForumUtil.getArticleByID(articleID);
        if (article == null) {
            throw new MissingDataException("could not find the specified article");
        }
        page.expose("article", article);
    }

    /**
     * Start a new discussion.
     */
    public void execNewDiscussion() throws IOException {
        page = getPage("post.nhtml");
        String forumID = getParameter("forum_id");
        Article article = ForumUtil.newArticle(forumID, null);
        page.expose("article", article);
    }

    /**
     * Contribute to an ongoing discussion.
     */
    public void execReply() throws IOException {
        page = getPage("post.nhtml");
        String articleID = getParameter("article_id");
        Article parent = ForumUtil.getArticleByID(articleID);
        if (parent  == null) {
            throw new MissingDataException("could not find the specified article");
        }
        Article article = ForumUtil.newArticle(null, parent);
        page.expose("article", article);
        page.expose("why", "reply");
    }

    public void execProcessNew() throws IOException {
        if (!checkSessionInfo())
            return;
        page = getPage("thanks.nhtml");
        DataRegistry registry = DataRegistryImpl.getDefaultRegistry();
        Article article = (Article) registry.getExemplar("forum_article");
        DataUtil.fillRecordFromServletRequest(article, request);
        article.set("author_id", userID);
        ForumUtil.storeArticle(article);
        page.expose("article", article);
        page.expose("why", "new article");
    }

    public void execProcessSignup() throws IOException {
        String loginID = getParameter("login_id");
        Record user = DataRegistryImpl.getDefaultRegistry().getExemplar("forum_user");
        DataUtil.fillRecordFromServletRequest(user, request);
        String password = Util.randomAlphaNumString(6);
        user.set("password", password);
        ForumUtil.getForumData().insert(user);
        page = getPage("thanks.nhtml");
        page.expose("why", "signup");
        page.expose("email", user.get("email"));
        if (!LOCAL_TESTING) {
            this.sendMail((String) user.get("email"), "Thanks for using our forums", 
                "Your login is: " + user.get("login_id") + " and your password is: " + 
                password);
        }
    }

    public void execLogin() throws IOException {
        if (hasValidLoginInfo()) {
            this.createNewSession();
            redirectAction("default");
        }
        else {
            page = getPage("error.nhtml");
            page.expose("why", "bad login");
        }
    }

    public void execLogout() throws IOException {
        if (session != null) {
            session.invalidate();
            session = null;
            sessionID = null;
            userID = null;
        }
        page = getPage("thanks.nhtml");
        page.expose("why", "logout");
    }

    protected boolean hasValidLoginInfo() throws IOException {
        String loginID = getParameter("login_id");
        String password = getParameter("password");
        Record user = ForumUtil.getUser(loginID);
        if (user == null || !password.equals(user.get("password"))) {
            return false;
        }
        else {
            this.userID = loginID;
            return true;
        }
    }

    protected void unauthorized() throws IOException {
        page = getPage("error.nhtml");
        page.expose("why", "unauthorized");
    }

    public void execSignup() throws IOException {
        page = getPage("signup.nhtml");
    }

    protected void exposeDefaultVariables() throws IOException {
        super.exposeDefaultVariables();
        page.expose("sessionID", sessionID);
        page.expose("userID", userID);
        page.expose("forums", ForumUtil.getForums());
        exposeActionURL("reply");
        exposeActionURL("signup");
        exposeActionURL("login");
        exposeActionURL("logout");
        exposeActionURL("processSignup");
    }

    protected void recover(IOException e) throws IOException {
        boolean ok = false;
        try {
            throw e;
        }
        catch (DataException de) {
            ok = true;
            String error = de.getLocalizedMessage();
            Record rec = de.getRecord();
            if (action.equals("processSignup")) {
                page = getPage("signup.nhtml");
            }
            else if (action.equals("processNew") || action.equals("reply")) {
                if (rec instanceof Article) {
                    page = getPage("post.nhtml");
                    page.expose("article", rec);
                }
            }
            else
                ok = false;
            if (page != null) {
                page.expose("error", de.getLocalizedMessage());
            }
        }
        if (!ok) 
            throw e;
    }

    protected boolean isAdmin() {
        return ADMIN_USER.equals(userID);
    }

    private void sendMail(String to, String subject, String message) throws IOException {
        String mailServer = config.getInitParameter("MAIL_SERVER");
        String from = config.getInitParameter("ADMIN_EMAIL");
        SmtpClient client = new SmtpClient(mailServer);
        client.from(from);
        client.to(to);
        PrintStream ps = client.startMessage();
        ps.println("To: " + to);
        ps.println("Subject:  " + subject);
        ps.println(message);
        client.closeServer();
    }

    static final boolean LOCAL_TESTING = false;
    // hard-coded here for the moment.
    static final String ADMIN_USER = "admin";
}   
